@@ -1,14 +1,16 @@
 # -*- coding: utf-8 -*-
 from __future__ import division
-from brightway2 import config, databases, methods, Database, Method, \
-    JsonWrapper, set_data_dir, bw2setup
-from bw2analyzer import DatabaseExplorer, SerializedLCAReport
+from .jobs import JobDispatch, InvalidJob
+from .utils import get_job_id, get_job, set_job_status, json_response, \
+    get_dynamic_media_folder
+from bw2analyzer import DatabaseExplorer, SerializedLCAReport, DatabaseHealthCheck
 from bw2calc.speed_test import SpeedTest
+from bw2data import config, databases, methods, Database, Method, \
+    JsonWrapper, set_data_dir, bw2setup
 from bw2data.io import Ecospold1Importer, EcospoldImpactAssessmentImporter
 from flask import url_for, render_template, request, redirect, abort
-from jobs import JobDispatch, InvalidJob
+from stats_arrays import uncertainty_choices
 from urllib import unquote as _unquote
-from utils import get_job_id, get_job, set_job_status, json_response
 import multiprocessing
 import os
 import urllib2
@@ -183,6 +185,11 @@ def import_method():
 
 @bw2webapp.route('/')
 def index():
+    try:
+        import bw2search
+        search_allowed = True
+    except ImportError:
+        search_allowed = False
     if config.is_temp_dir and not config.p.get("temp_dir_ok", False):
         return redirect(url_for('start'))
     dbs = [{
@@ -204,7 +211,7 @@ def index():
         'methods': JsonWrapper.dumps(ms),
         'config': config
         }
-    return render_template("index.html", **context)
+    return render_template("index.html", search_allowed=search_allowed, **context)
 
 
 @bw2webapp.route('/ping', methods=['GET'])
@@ -237,6 +244,37 @@ def speed_test():
     st = SpeedTest()
     return str(250 * int(40 * st.ratio()))
 
+
+#################
+### Searching ###
+#################
+
+
+@bw2webapp.route('/search')
+def whoosh_index():
+    try:
+        from bw2search import Searcher
+    except ImportError:
+        print u"Search functionality requires `bw2search`."
+        abort(500)
+    return render_template('search.html')
+
+@bw2webapp.route('/search_request', methods=["POST"])
+def whoosh_search():
+    try:
+        from bw2search import Searcher
+    except ImportError:
+        abort(500)
+
+    try:
+        request_data = JsonWrapper.loads(request.data)
+    except:
+        abort(400)
+
+    data = {'results': Searcher().search(request_data['search_string'])}
+    return json_response(data)
+
+
 ##############################
 ### Databases and datasets ###
 ##############################
@@ -269,6 +307,7 @@ def database_explorer(name):
         name=name,
         depends=depends,
         data=JsonWrapper.dumps(json_data),
+        health_check_url=url_for('database_health_check', database=name),
         backup_url = url_for('backup_database', database=name),
         delete_url = url_for('delete_database', database=name),
     )
@@ -376,6 +415,42 @@ def json_editor(database, code):
         return abort(404)
     return render_template("jsoneditor.html", jsondata=JsonWrapper.dumps(data))
 
+
+@bw2webapp.route("/database/<database>/health-check")
+def database_health_check(database):
+    if database not in databases:
+        return abort(404)
+    data = Database(database).load()
+    np = len(data)
+
+    def reformat(key, score=None):
+        ds = data[key]
+        return {
+            'name': ds.get('name', "Unknown"),
+            'categories': ",".join(ds.get('categories', [])),
+            'location': ds.get('location', ''),
+            'unit': ds.get('unit', ''),
+            'url': url_for('activity_dataset-canonical', database=key[0], code=key[1]),
+            'score': score
+        }
+
+    dhc = DatabaseHealthCheck(database).check(get_dynamic_media_folder())
+    dhc['pr'] = JsonWrapper.dumps([reformat(key, score * np) for score, key in dhc['pr'][:20]])
+    dhc['uncertainty'] = sorted([
+        (uncertainty_choices[key].description, value['total'], value['bad'])
+        for key, value in dhc['uncertainty'].items()
+        if value['total']
+    ], key=lambda x: x[1], reverse=True)
+    for obj in ('mo', 'me', 'sp'):
+        if dhc[obj]:
+            dhc[obj] = JsonWrapper.dumps([reformat(key, number) for key, number in dhc[obj]])
+    for obj in ('nsp', 'ob'):
+        if dhc[obj]:
+            dhc[obj] = JsonWrapper.dumps([reformat(key) for key in dhc[obj]])
+
+    return render_template("health-check.html", database=database, **dhc)
+
+
 ###########
 ### LCA ###
 ###########
